function [p_vals, R2, param_p, SS, dof, MS, f_vals] = rsanova(y, X)
% RSANOVA - Perform ANalysis Of VAriance for linear regression. Version 0.1.16.
%
% If replicated experiments exist in the data, then lack of fit is automatically calculated.
%
% The syntax is
% [p_vals, R2, param_p, SS, dof, MS, f_vals] = rsanova(y, X)
%
% Arguments
% y is an N by 1 column vector of data to fit.
% X is an N by a matrix containing the values of a predictors used in the regression, including a possible constant 1 column.
%
% Return values
% p_vals is a row vector of p-values corresponding to the null hypotheses [overall_regression_insignificant, model_fits_data]. The latter is only included if replicate experiments exist. If X has only one column then the former is also omitted.
% R2 is the unadjusted coefficient of determination SSR/SST.
% param_p is a row vector of p-values for the null hypotheses H0: "beta_i=0" with the alternative H1: "beta_i!=0", where i runs through all the parameters: 1 <= i <= size(X,2).
% SS is a row vector of sums of squares. The first elements correspond to [total, regression, residual]. If replicate experiments exist (and also SSLOF have at least one degree of freedom) then SS further contains [Lack_of_Fit, pure_error].
% dof is a row vector of degrees of freedom. Order is the same as for SS.
% MS is a row vector of degrees of mean squares. Order is the same as for SS.% cor are the correlations corresponding to the p values in p_cor.
% f_vals is a row vector of the F-test statistics corresponding to p_vals.
%
% References
% For background, see Sections 2.4 and 2.7 in Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009.


% Copyright (C) 2015 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2015


% Check input
if nargin ~= 2,
    error('Try help rsanova for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsanova for instructions.');
end
if cond(X'*X) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix X does not have linearly dependent columns, try standardizing the factor data.');
end

% Still check that ANOVA is possible
b=X\y;
pred=X*b;
r=y-pred;
SSE=r'*r;
if SSE < 1e-6,
    error('Your model seems to fit the data perfectly, then no ANOVA is possible. If the equation X*b=y is in fact overdetermined, try normalizing the factor data.');
end

% Perform Anova
[N,a]=size(X);
dfE=N-a;
MSE=SSE/dfE;
MST=var(y);
dfT=N-1;
SST=MST*dfT;
SSR=SST-SSE;
dfR=dfT-dfE;
if dfR>0
  MSR=SSR/dfR;
else
  MSR=[];
end
if dfE>0 && dfR>0
  f_sig=MSR/MSE;
  p_sig=1-fcdf(f_sig,dfR,dfE);
else
  f_sig=[];
  p_sig=[];
end

% Put results into return vectors
SS=[SST SSR SSE];
MS=[MST MSR MSE];
dof=[dfT dfR dfE];
p_vals=[p_sig];
R2=SSR/SST;
f_vals=[f_sig];
T=0;

% If repeat experiments exist and all degrees of
% freedom are positive, split residual sum of
% squares into pure error and lack of fit.
[~, ~, ic] = unique(X,'rows');
m=length(unique(ic));
dfPE=N-m;
dfLOF=dfE-dfPE;

if dfPE>0 && dfLOF>0,
    Im=eye(m);
    T=Im(ic,:);
    rT=y-T*(T\y);
    SSPE=rT'*rT;
    MSPE=SSPE/dfPE;
    SSLOF=SSE-SSPE;
    MSLOF=SSLOF/dfLOF;
    f_lof=MSLOF/MSPE;
    p_lof=1-fcdf(f_lof,dfLOF,dfPE);
    
    SS=[SS SSLOF SSPE];
    MS=[MS MSLOF MSPE];
    dof=[dof dfLOF dfPE];
    p_vals=[p_vals p_lof];
    f_vals=[f_vals f_lof];
end

for i=1:a,
  Ai=zeros(1,a);
  Ai(i)=1;
  param_p(i)=rslinhyp(y, X, 0, Ai);
end

end
