function [z, eigenv, pred, confint_mean] = rsextreme(y, D, alfa)
% RSEXTREME - Fit a full quadratic model and describe it's stationary point. Version 0.1.21.
%
% The syntax is
% [z, eigenvals, pred, confint_mean] = rsextreme(y, D, [alpha])
%
% Arguments
% y is an N by 1 column vector of measured response data.
% D is an N by k design matrix with factor data.
% [alpha] is a voluntary parameter determining what 1-alpha confidence interval is returned. If omitted then alpha=0.05 is assumed.
%
% Return values
% z is the location of the stationary point
% eigenv are the eigenvalues of the half Hessian B in the fitted model y=b0+x*b1+x*B*x'. All positive iff we have a local minimum, all negative iff we have a local maximum. If some eigenvalue is close to zero, then we have a ridge system.
% pred is the response predicted by the model at the stationary point
% 
%
% References
% Myers, Montgomery & Anderson-Cook: Response Surface Methodology, 3rd ed., Wiley, 2009, Section 6.3.


% Copyright (C) 2016 Mikael Kurula <mkurula@abo.fi>
%
% This program is free software; you can redistribute it and/or modify it under
% the terms of the GNU General Public License as published by the Free Software
% Foundation; either version 3 of the License, or (at your option) any later
% version.
%
% This program is distributed in the hope that it will be useful, but WITHOUT
% ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
% FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
% details.
%
% You should have received a copy of the GNU General Public License along with
% this program; if not, see <http://www.gnu.org/licenses/>.
%
% Author: Mikael Kurula <mkurula@abo.fi>, 2016


% Check input
if nargin < 2 || nargin > 3
    error('Try help rsextreme for instructions.') ;
end
if size(y,2) > 1,
    error('y has more than one column. Try help rsextreme for instructions.');
end
if cond(D'*D) > 1e10,
    warning('The normal equation is ill-conditioned. If the matrix D does not have linearly dependent columns, try standardizing the factor data.');
end

if nargin==2 
  alfa=0.05;
end

% Extreme value analysis
k=size(D,2);
X=rsquad(D);
b=X\y;
b0=b(1);
b1=b(2:(k+1));
b(1:(k+1))=[];
F=zeros(k,k);
for j=0:(k-1)
  F=F+diag(b(1:(k-j)),j);
  b(1:(k-j))=[];
end
B=(F+F')/2;
eigenv=eig(B)';
z=-B^-1*b1/2;
z=z';

% Prediction at stationary point
x=rsquad(z);
[pred, confint_mean]=rspredict(y,X,x,alfa);

end